# ScanAlyze
# ScanAlyze
# http://rana.lbl.gov/EisenSoftware.htm
# http://rana.lbl.gov/manuals/ScanAlyzeDoc.pdf
# FLAG may differ between hybridisations
# example files:
# fileNames <- c("~/bunessHerkules/link_to_whuberHerkules/PMA/03.2003/lymphoma/data/lc7b057rex.DAT", "~/bunessHerkules/link_to_whuberHerkules/PMA/03.2003/lymphoma/data/lc7b047rex.DAT", "~/bunessHerkules/link_to_whuberHerkules/PMA/03.2003/lymphoma/data/lc7b056rex.DAT", "~/bunessHerkules/link_to_whuberHerkules/PMA/03.2003/lymphoma/data/lc7b070rex.DAT" )

      
# /**
#
# \name{readImageFiles}
#
# \title{readImageFiles}
#
# \alias{readImageFiles}
#
# \keyword{}
#
# \description{
#             All image analysis files are expected to correspond
#             to the same type of microarray.
#
# }
#
#
# \arguments{
#  \item{fileNames}{ vector of character strings; default missing }
#  \item{spotAnnoColumns}{ vector of character strings;
#        the column names of the image analysis data file,
#        which are common to all microarrays; default missing }
#  \item{dataColumns}{ named vector of character strings;
#         the column names of the image analysis data file,
#         which contain the raw intensities values of each spot; 
#         if supplied names(dataColumns) must contain:
#         c("Cy3","Cy3Background","Cy5","Cy5Background");
#         default missing
#       }
#  \item{type}{character string;
#              possible values: "GenePix" and "ScanAlyze";
#              default: "GenePix" }
#  \item{verbose}{logical; default \code{TRUE}}
# }
#
# \value{object of class \code{arrayData}}
#
# \author{Andreas Buness <a.buness@dkfz.de>}
#
# */
readImageFiles <- function(fileNames,
                           spotAnnoColumns,
                           dataColumns,
                           type = "GenePix",
                           verbose = TRUE){

  if( verbose ){cat(" reading image files \n")}
  
  if( length(type) != 1 ){
    stop(paste("type:",type," must be of length one in readImageFiles",sep=""))
  }
  if( ! type %in% c("GenePix","ScanAlyze") ){
    stop(paste(" wrong type:",type," in readImageFiles",sep=""))
  }
  
  if( type == "GenePix" ){
    if( verbose ){cat(" type: GenePix \n")}
    if( missing(spotAnnoColumns) ){
      spotAnnoColumns <-  c("Block", "Column", "Row", "Name", "ID")
    }
    #       Note:
    #         Cy3 - 532 - green
    #         Cy5 - 635 - red
    if( missing(dataColumns) ){
      dataColumns <- c("F532.Median", "B532.Median", "F635.Median", "B635.Median")
      names(dataColumns) <- c("Cy3","Cy3Background","Cy5","Cy5Background")
    }
  }else if( type == "ScanAlyze" ){
    if( verbose ){cat(" type: ScanAlyze \n")}
    if( missing(spotAnnoColumns) ){
      spotAnnoColumns <- c( "HEADER", "SPOT", "GRID", "ROW", "COL" )
    }
    if( missing(dataColumns) ){
      # median values, channel 1 green, channel 2 red
      dataColumns <- c("CH1I", "CH1B", "CH2I", "CH2B")
      names(dataColumns) <- c("Cy3","Cy3Background","Cy5","Cy5Background")
    }
  }else{
    stop(paste(" wrong type:",type," in readImageFiles",sep=""))
  }

  if( verbose ){cat(paste(" using dataColumns:\n"));print(dataColumns)}
  if( verbose ){cat(paste(" using spotAnnoColumns: ", paste(spotAnnoColumns, collapse=","),"\n"))}
  
  
  if( ! (all(names(dataColumns) %in%  c("Cy3","Cy3Background","Cy5","Cy5Background") ) ) ){
    stop(paste(" wrong names(dataColumns):",paste(names(dataColumns),collapse=",")," in readImageFiles",sep=""))
  }
  if( missing(fileNames) ){
    stop(" no fileNames in readImageFiles \n")
  }
  if( length(fileNames) < 1 ){
    stop(" length(fileNames) < 1 in readImageFiles \n")
  }
  
  nrOfFiles <- length(fileNames)

  for (i in (1:nrOfFiles)){
    
    fn <- fileNames[i]
    if( verbose ){cat(" reading", fn, "\n")}

    # read file fn and create data.frame dat
    if( type == "GenePix" ){
      
      txt <- scan(file=fn,what="character",sep="\n")
      skip <- grep("Block...Column", txt) 
      stopifnot(is.numeric(skip) && length(skip)==1)
      dat <- read.table(fn, sep='\t', header=TRUE, skip=skip-1, as.is=TRUE)

    }else if( type == "ScanAlyze" ){

      dat <- read.table(file=fn, sep="\t", header=TRUE, as.is=TRUE)
      
    }else{
      
      stop(paste(" wrong type:",type," in readImageFiles",sep=""))
        
    }
            
    nc <- spotAnnoColumns[!(spotAnnoColumns %in% colnames(dat))]
    if(length(nc)>0){
      stop(paste("spotAnnoColumns", nc, "not found in data file: ",fn))
    }
    nc <- dataColumns[!(dataColumns %in% colnames(dat))]
    if(length(nc)>0){
      stop(paste("dataColumns", nc, "not found in data file: ",fn))
    }
    
    if(i==1){
      # array dimensions of qua [ nr of genes, 4, nr of hybridisations ]
      qua <- array(NA, dim=c(nrow(dat), 4, nrOfFiles))
      dimnames(qua) <- list( NULL, c("Cy3", "Cy3Background", "Cy5", "Cy5Background"), fileNames ) 
      # annotation information for all genes
      spotanno <-  dat[,spotAnnoColumns]
    }else{
        # check consistency of gpr information, i.e. the same slide type
      stopifnot(all(dat[,spotAnnoColumns]==spotanno))
    }
      
    qua[,"Cy3",i] <- dat[,dataColumns["Cy3"]] 
    qua[,"Cy3Background",i] <-dat[,dataColumns["Cy3Background"]] 
    qua[,"Cy5",i] <- dat[,dataColumns["Cy5"]] 
    qua[,"Cy5Background",i] <- dat[,dataColumns["Cy5Background"]] 
    
  } # end of for i in nrOfFiles

  return(new("arrayData",intensities=qua,spotattr=spotanno))
  
}
