# TO DO more testing, e.g. rawCy3Relative

# /**
#
# \name{writeToFile}
#
# \title{writeToFile}
#
# \alias{writeToFile}
#
# \keyword{}
#
# \description{ Generation of a tab-deliminated output file }
#
# \value{}
#
#
# \arguments{
#  \item{exprSetArrayObject}{exprSetArrayObject}
#  \item{arrayDataObject}{arrayDataObject}
#  \item{rowSelection}{vector of indexes; default: missing}
#  \item{slideNumberColumn}{character string; by default "slideNumber", must refer to a valid column in getHybAttr(arrayDataObject)}
#     }
#  \item{channels}{vector of character strings; by default: c("both");
#       possible values are:
#       "both", "Cy3", "Cy5", "rawCy3", "rawCy5", "rawCy3Relative", "rawCy5Relative" }
#  \item{fileName}{character string; default "normalisedData.txt"}
#  \item{savePath}{character string; default ""}
#  \item{fullOutput}{logical;adds phenoData information, by default \code{TRUE}}
#  \item{coding}{logical;adds integer coded phenoData information if \code{fullOutput} is \code{TRUE}}
# }
# 
# \details{}
#
# \seealso{\code{}}
#
# \examples{
#	}
#
# \keyword{}
#
# \author{Andreas Buness <a.buness@dkfz.de>}
# */


writeToFile <- function(exprSetArrayObject,
                        arrayDataObject,
                        rowSelection,
                        slideNumberColumn="slideNumber",
                        channels=c("both"),
                        fileName="normalisedData.txt",
                        savePath="",
                        fullOutput=TRUE,
                        coding=FALSE){

  if( ! is.logical(fullOutput) ){
    cat(" fullOutput must be of type logical in writeToFile \n")
    return()
  }
  if( ! is.logical(coding) ){
    cat(" coding must be of type logical in writeToFile \n")
    return()
  }
  if( coding & !fullOutput ){
    cat(" coding only applies if fullOutput==TRUE in writeToFile \n")
  }
  if( missing(exprSetArrayObject) ){
    cat(" exprSetArrayObject is missing in writeToFile \n")
    return()
  }
  if( class(exprSetArrayObject) != "exprSetArray" ){
    cat(paste(" invalid class:", class(exprSetArrayObject)," of exprSetArrayObject in writeToFile \n",sep=""))
    return()
  } 
  if( missing(arrayDataObject) ){
    cat(" arrayDataObject is missing in writeToFile \n")
    return()
  }
  if( class(arrayDataObject) != "arrayData" ){
    cat(paste(" invalid class:", class(arrayDataObject)," of arrayDataObject in writeToFile \n",sep=""))
    return()
  } 
    
  fullFileName <- file.path(savePath,fileName)
  
  channelValues <- c("both", "Cy3", "Cy5", "rawCy3", "rawCy5", "rawCy3Relative", "rawCy5Relative")
  if( ! all(channels %in% channelValues) ){
    cat(paste(" wrong channels:",paste(channels,collapse=","),"\n"," all possible valid items are as follows: ",paste(channelValues,collapse=","),sep=""), "\n end of wrong channels in writeToFile \n")
  }

  
  slides <- getHybAttr(arrayDataObject)[,slideNumberColumn]
  spottAttr <- getSpotAttr(arrayDataObject) # data.frame
  nrOfSpotRows <- dim(spottAttr)[1]
  nrOfSpotColumns <- dim(spottAttr)[2]
  nrOfExprRows <- dim(exprs(exprSetArrayObject))[1]
  nrOfExprSlides <- dim(exprs(getExprSetDiff(exprSetArrayObject)))[2]
  nrOfChannels <- length(channels)
  nrOfVarLabels <-  length(unlist(varLabels(phenoData(exprSetArrayObject))))
  
  if( nrOfSpotRows != nrOfExprRows ){
    cat(paste(" rows in arrayDataObject:",nrOfSpotRows," and rows in exprSetArrayObject:",nrOfSpotRows," do not match in writeToFile \n",sep=""))
    return()
  }
  if( length(slides) != nrOfExprSlides ){
    cat(paste(" number of slides in arrayDataObject:", length(slides)," and number of slides in exprSetArrayObject:",nrOfExprSlides," do not match in writeToFile \n",sep=""))
    return()
  }

  #############
  allValueMatrixCols <- NULL
  allValueMatrix <- NULL
  emptyColumn <- matrix(data="",nrow=nrOfSpotRows,ncol=1)
  emptySpotBlock <- matrix(data="",nrow=nrOfVarLabels,ncol=nrOfSpotColumns)
  fullOutputFrame <- emptySpotBlock
  fullCodingOutputFrame <- emptySpotBlock
  
  for( channel in channels){
    
    # generation of a value matrix
    if( channel == "both" ){
      tmpPhenoData <- phenoDataSlide(exprSetArrayObject)
      valueMatrix <- exprs(getExprSetDiff(exprSetArrayObject)) # matrix
      valueMatrixCols <- paste("differenceOf",slides,sep="")
    }else if( channel == "Cy3" ){
      tmpPhenoData <- phenoDataCy3(exprSetArrayObject)
      valueMatrix <- exprs(getExprSetCy3(exprSetArrayObject)) # matrix
      valueMatrixCols <- paste("Cy3Of",slides,sep="")
    }else if( channel == "Cy5" ){
      tmpPhenoData <- phenoDataCy5(exprSetArrayObject)
      valueMatrix <- exprs(getExprSetCy5(exprSetArrayObject)) # matrix
      valueMatrixCols <- paste("Cy5Of",slides,sep="")
    }else if( channel == "rawCy3" ){
      tmpPhenoData <- phenoDataCy3(exprSetArrayObject)
      valueMatrix <- getIntensities(arrayDataObject)[,"Cy3",]
      valueMatrixCols <- paste("rawCy3Of",slides,sep="")
    }else if( channel == "rawCy5" ){
      tmpPhenoData <- phenoDataCy5(exprSetArrayObject)
      valueMatrix <- getIntensities(arrayDataObject)[,"Cy5",]
      valueMatrixCols <- paste("rawCy5Of",slides,sep="")
    }else if( channel == "rawCy3Relative" ){
      tmpPhenoData <- phenoDataCy3(exprSetArrayObject)
      valueMatrix <- aperm(simpleApply(getIntensities(arrayDataObject)[,"Cy3",],dimension=2,func=rank,funcResultDimensionality=nrOfExprRows)/nrOfExprRows,c(2,1))
      valueMatrixCols <- paste("rawCy3RelativeOf",slides,sep="")
    }else if( channel == "rawCy5Relative" ){
      tmpPhenoData <- phenoDataCy5(exprSetArrayObject)
      valueMatrix <- aperm(simpleApply(getIntensities(arrayDataObject)[,"Cy5",],dimension=2,func=rank,funcResultDimensionality=nrOfExprRows)/nrOfExprRows,c(2,1))
      valueMatrixCols <- paste("rawCy5RelativeOf",slides,sep="")
    }else{
      stop(" unknown channel in writeToFile ")
    }

    # phenoData annotation
    varLabelsVec <- unlist(varLabels(tmpPhenoData))    
    dim(varLabelsVec) <- c(length(varLabelsVec),1)
    pDataObject <- pData(tmpPhenoData)  
    fullOutputFrame <- cbind(fullOutputFrame,varLabelsVec, t(pDataObject))

    # integer coded phenoData annotation
    pDataCodes <- apply(pDataObject,2, function(x) codes(as.factor(x)))
    fullCodingOutputFrame <- cbind(fullCodingOutputFrame, varLabelsVec, t(pDataCodes))
    
    allValueMatrixCols <- c(allValueMatrixCols, "", valueMatrixCols)
    allValueMatrix <- cbind(allValueMatrix, emptyColumn, valueMatrix)
    
    
  }# end of for channel in channels loop
    

  # if fullOutput == TRUE : generation of a header
  # which contains the phenoData information
  if( fullOutput ){
    APPEND <- TRUE

    write.table(fullOutputFrame, file=fullFileName, quote=FALSE, sep="\t",row.names=FALSE, col.names=FALSE)
    
     # integer coded pData
    if( coding ){
      
      write.table(fullCodingOutputFrame, file=fullFileName, quote=FALSE, sep="\t", row.names=FALSE, col.names=FALSE, append=TRUE)
      
    }
  }else{
    APPEND <- FALSE
  }

  # defines the first line of the data block which
  # contains the spottAttr followed by an empty columm
  # followed by the value matrix

  outputColNames <- c(colnames(spottAttr),allValueMatrixCols)
  dim(outputColNames) <- c(1,length(outputColNames))
  if( missing(rowSelection) ){
    rowSelection <- 1:nrOfSpotRows
  }

  outputFrame <-  cbind(spottAttr, allValueMatrix)[rowSelection,]
  
  write.table(outputColNames, file=fullFileName, quote=FALSE, sep="\t",row.names=FALSE, col.names=FALSE, append=APPEND)
  write.table(outputFrame, file=fullFileName, quote=FALSE, sep="\t",row.names=FALSE, col.names=FALSE, append=TRUE)

}


